#####################################################################################
###		Code to calculate prediction error of height diameter models	#####
#		Used in Sullivan et al. 					#####
# Field methods for sampling tree height for tropical forest biomass estimation	#####	
#		in Methods in Ecology and Evolution				#####
###		Author: Martin Sullivan						#####
###		Date: 13 July 2017						#####
#####################################################################################

#Load required package
library(BIOMASS)

#####################################################################################
#		Functions needed						#####
#####################################################################################

#Estimate AGB using Chave et al. 2014 equation
#dbh = diameter at breast height (mm)
#H = height (m)
#WD = wood density
agb.mod<-function(dbh,H,WD){
	0.0673 *(WD * (dbh/10)^2* H)^0.976/1000
}

#Calculate prediction error from observed and predicted tree height
#obs = observed (measured) height
#pred = predicted height
#D = diameter at breast height
#WD = wood density
#AGBobs = aboveground biomass estimated from observed height
eval.metrics<-function(obs,pred,D,WD,AGBobs){
	rmse<-sqrt(mean((obs-pred)^2))
	bias<-((mean(pred)-mean(obs))/mean(obs))*100
	agb.pred<-agb.mod(D,pred,WD)
	agb.error<-((sum(agb.pred)-sum(AGBobs))/sum(AGBobs))*100
	res<-c(rmse,bias,agb.error)
	res
}

#Modified version of modelHD from the BIOMASS package to remove limitations of minimum sample size model can be fitted to
#Please cite the BIOMASS package if you use this code!!!
modelHD2<-function (D, H, method = NULL, useWeight = FALSE, drawGraph = FALSE) 
{
    nbNonNA <- sum(!is.na(H))
    if (nbNonNA < 10) 
        stop(paste("The data has not enough height data (less than 10 non NA)"))
    Hdata <- data.frame(H, D)
    names(Hdata) <- c("H", "D")
    Hdata <- na.omit(Hdata)
    weight <- NULL
    D_Plot <- seq(from = floor(min(Hdata$D)), to = ceiling(max(Hdata$D)), 
        0.5)
    if (useWeight == TRUE) 
        weight <- (Hdata$D^2) * Hdata$H
    if (!is.null(method)) {
        RSElog <- NULL
        if (grepl("log", method)) {
            modSelected <- loglogFunction(Hdata, method)
            RSElog <- summary(modSelected)$sigma
            coeff <- summary(modSelected)$coefficients
            if (method == "log1") {
                Hpredict_plot <- exp(coeff[1] + 0.5 * RSElog^2 + 
                  coeff[2] * log(D_Plot))
                Hpredict <- exp(coeff[1] + 0.5 * RSElog^2 + coeff[2] * 
                  log(Hdata$D))
            }
            if (method == "log2") {
                Hpredict_plot <- exp(coeff[1] + 0.5 * RSElog^2 + 
                  coeff[2] * log(D_Plot) + coeff[3] * log(D_Plot)^2)
                Hpredict <- exp(coeff[1] + 0.5 * RSElog^2 + coeff[2] * 
                  log(Hdata$D) + coeff[3] * log(Hdata$D)^2)
            }
            if (method == "log3") {
                Hpredict_plot <- exp(coeff[1] + 0.5 * RSElog^2 + 
                  coeff[2] * log(D_Plot) + coeff[3] * log(D_Plot)^2 + 
                  coeff[4] * log(D_Plot)^3)
                Hpredict <- exp(coeff[1] + 0.5 * RSElog^2 + coeff[2] * 
                  log(Hdata$D) + coeff[3] * log(Hdata$D)^2 + 
                  coeff[4] * log(Hdata$D)^3)
            }
        }
        if (method == "weibull") {
            modSelected <- weibullFunction(Hdata, weight)
            coeff <- summary(modSelected)$coefficients
            a <- coeff[1]
            b <- coeff[2]
            c <- coeff[3]
            Hpredict_plot <- a * (1 - exp(-(D_Plot/b)^c))
            Hpredict <- a * (1 - exp(-(Hdata$D/b)^c))
        }
        if (method == "michaelis") {
            modSelected <- michaelisFunction(Hdata, weight)
            coeff <- summary(modSelected)$coefficients
            A <- coeff[1]
            B <- coeff[2]
            Hpredict_plot <- SSmicmen(D_Plot, A, B)
            Hpredict <- SSmicmen(Hdata$D, A, B)
        }
        if (drawGraph == TRUE) {
            par(mar = c(5, 5, 3, 3))
            plot(Hdata$D, Hdata$H, pch = 20, cex = 0.5, col = "grey50", 
                log = "xy", las = 1, xlab = "D (cm)", ylab = "H (m)", 
                cex.lab = 1.8, cex.axis = 1.5, main = paste("Selected model : ", 
                  method), cex.main = 2)
            lines(D_Plot, Hpredict_plot, lwd = 2, col = "blue")
            legend("bottomright", c("Data", "Model selected"), 
                lty = c(3, 1), lwd = c(3, 3), col = c("grey", 
                  "blue"), cex = 1.5)
        }
    }
    else {
        mod_log1 <- loglogFunction(Hdata, method = "log1")
        RSElog <- summary(mod_log1)$sigma
        coeff <- summary(mod_log1)$coefficients
        Hpredict_log1_plot <- exp(coeff[1] + 0.5 * RSElog^2 + 
            coeff[2] * log(D_Plot))
        Hpredict_log1 <- exp(coeff[1] + 0.5 * RSElog^2 + coeff[2] * 
            log(Hdata$D))
        mod_log2 <- loglogFunction(Hdata, method = "log2")
        RSElog <- summary(mod_log2)$sigma
        coeff <- summary(mod_log2)$coefficients
        Hpredict_log2_plot <- exp(coeff[1] + 0.5 * RSElog^2 + 
            coeff[2] * log(D_Plot) + coeff[3] * log(D_Plot)^2)
        Hpredict_log2 <- exp(coeff[1] + 0.5 * RSElog^2 + coeff[2] * 
            log(Hdata$D) + coeff[3] * log(Hdata$D)^2)
        mod_log3 <- loglogFunction(Hdata, method = "log3")
        RSElog <- summary(mod_log3)$sigma
        coeff <- summary(mod_log3)$coefficients
        Hpredict_log3_plot <- exp(coeff[1] + 0.5 * RSElog^2 + 
            coeff[2] * log(D_Plot) + coeff[3] * log(D_Plot)^2 + 
            coeff[4] * log(D_Plot)^3)
        Hpredict_log3 <- exp(coeff[1] + 0.5 * RSElog^2 + coeff[2] * 
            log(Hdata$D) + coeff[3] * log(Hdata$D)^2 + coeff[4] * 
            log(Hdata$D)^3)
        mod_wei <- weibullFunction(Hdata, weight)
        coeff <- summary(mod_wei)$coefficients
        a <- coeff[1]
        b <- coeff[2]
        c <- coeff[3]
        Hpredict_wei_plot <- a * (1 - exp(-(D_Plot/b)^c))
        Hpredict_wei <- a * (1 - exp(-(Hdata$D/b)^c))
        mod_mich <- michaelisFunction(Hdata, weight)
        coeff <- summary(mod_mich)$coefficients
        A <- coeff[1]
        B <- coeff[2]
        Hpredict_mich_plot <- SSmicmen(D_Plot, A, B)
        Hpredict_mich <- SSmicmen(Hdata$D, A, B)
        par(mar = c(5, 5, 3, 3))
        plot(Hdata$D, Hdata$H, pch = 20, cex = 0.5, col = "grey50", 
            log = "xy", las = 1, xlab = "D (cm)", ylab = "H (m)", 
            cex.lab = 1.8, cex.axis = 1.5, main = "Model comparison", 
            cex.main = 2)
        lines(D_Plot, Hpredict_log1_plot, lwd = 2, col = "blue")
        lines(D_Plot, Hpredict_log2_plot, lwd = 2, col = "green")
        lines(D_Plot, Hpredict_log3_plot, lwd = 2, col = "red")
        lines(D_Plot, Hpredict_wei_plot, lwd = 2, col = "orange")
        lines(D_Plot, Hpredict_mich_plot, lwd = 2, col = "purple")
        legend("bottomright", c("Log 1", "Log 2", "Log 3", "Weibull", 
            "Michaelis"), lty = c(1, 1, 1, 1, 1), lwd = c(2, 
            2, 2, 2, 2), cex = 1.5, col = c("blue", "green", 
            "red", "orange", "purple"))
        cat("Which model would you like to select to model your data ? \n \n")
        cat("1 : Log 1 (blue) \n")
        printFitData(Hdata$H, Hpredict_log1, mod_log1)
        cat("2 : Log 2 (green) \n")
        printFitData(Hdata$H, Hpredict_log2, mod_log2)
        cat("3 : Log 3 (red) \n")
        printFitData(Hdata$H, Hpredict_log3, mod_log3)
        cat("4 : Weibull (orange) \n")
        printFitData(Hdata$H, Hpredict_wei, mod_wei)
        cat("5 : Michaelis - Menten (purple) \n")
        printFitData(Hdata$H, Hpredict_mich, mod_mich)
        ans <- scan(what = integer(), nmax = 1, quiet = TRUE)
        modSelected <- switch(ans, `1` = mod_log1, `2` = mod_log2, 
            `3` = mod_log3, `4` = mod_wei, `5` = mod_mich)
        Hpredict <- switch(ans, `1` = Hpredict_log1, `2` = Hpredict_log2, 
            `3` = Hpredict_log3, `4` = Hpredict_wei, `5` = Hpredict_mich)
        method <- switch(ans, `1` = "log1", `2` = "log2", `3` = "log3", 
            `4` = "weibull", `5` = "michaelis")
    }
    coeff <- summary(modSelected)$coefficients
    Residuals <- Hdata$H - Hpredict
    rSquared <- summary(modSelected)$r.squared
    formula <- summary(modSelected)$call
    RSE <- sqrt(sum(Residuals^2, na.rm = TRUE)/summary(modSelected)$df[2])
    output <- list(input = list(H = Hdata$H, D = Hdata$D), model = modSelected, 
        residuals = Residuals, coefficients = coeff, R.squared = rSquared, 
        formula = formula, method = method, predicted = Hpredict, 
        RSE = RSE)
    if (grepl("log", method)) 
        output$RSElog = RSElog
    return(output)
}

#Fit H-D models to training data, test on testing dataset
#train = subset of data used to train H-D models. Must contain columns D (diameter) and H (height)
#test = subset of data used to test H-D models. Must contain columns H, D, WD (wood density), AGBobs (AGB estimated from observed height), Hest (height estimated using regional model) and Hchave (height estimate using climate based model) 
#Plot = ID stamp to give output showing the plot used (i.e. ID column in dat) 
#ID = ID stamp to give output showing run ID
run.models<-function(train,test,Plot,ID){
	f1<-tryCatch(modelHD2(train$D,train$H,"weibull"),error=function(e)e)
	f2<-tryCatch(modelHD2(train$D,train$H,"michaelis"),error=function(e)e)
	f4<-tryCatch(modelHD2(train$D,train$H,"weibull",useWeight=TRUE),error=function(e)e)
	f5<-tryCatch(modelHD2(train$D,train$H,"michaelis",useWeight=TRUE),error=function(e)e)
	if(is(f1,"error")){
		p1<-NA
	}else{
		p1<-predictHeight(test$D,f1)
	}
	if(is(f2,"error")){
		p2<-NA
	}else{
		p2<-predictHeight(test$D,f2)
	}
		if(is(f4,"error")){
		p4<-NA
	}else{
		p4<-predictHeight(test$D,f4)
	}
	if(is(f5,"error")){
		p5<-NA
	}else{
		p5<-predictHeight(test$D,f5)
	}

	f3<-modelHD2(train$D,train$H,"log1")
	p3<-predictHeight(test$D,f3)

	result<-rbind(eval.metrics(test$H,p1,test$D,test$WD,test$AGBobs),
	eval.metrics(test$H,p2,test$D,test$WD,test$AGBobs),
	eval.metrics(test$H,p3,test$D,test$WD,test$AGBobs),
	eval.metrics(test$H,p4,test$D,test$WD,test$AGBobs),
	eval.metrics(test$H,p5,test$D,test$WD,test$AGBobs),
	eval.metrics(test$H,test$Hest,test$D,test$WD,test$AGBobs),
	eval.metrics(test$H,test$Hchave,test$D,test$WD,test$AGBobs))
	result<-as.data.frame(result)
	names(result)<-c("RMSE","Bias","AGB.diff")
	result$RMSE.order<-rank(result$RMSE)
	result$Method<-c("W","M","L","Wwt","Mwt","F","C")
	result$BestMod<-result$Method[result$RMSE.order==1]
	result$Sample.size<-nrow(train)
	result$RunID<-ID
	result$PlotID<-Plot
	result
}


#Fit H-D models with different sampling strategies - simplified version of run.models, with fewer different models and a sampling strategy ID stamp
samp.strat<-function(test,train,Plot,ID,strat){
	f1<-tryCatch(modelHD2(train$D,train$H,"weibull"),error=function(e)e)
	f2<-tryCatch(modelHD2(train$D,train$H,"michaelis"),error=function(e)e)
	if(is(f1,"error")){
		p1<-NA
	}else{
		p1<-predictHeight(test$D,f1)
	}
	if(is(f2,"error")){
		p2<-NA
	}else{
		p2<-predictHeight(test$D,f2)
	}

	result<-rbind(eval.metrics(test$Height,p1,test$D,test$WD,test$AGBobs),
	eval.metrics(test$Height,p2,test$D,test$WD,test$AGBobs))
	result<-as.data.frame(result)
	names(result)<-c("RMSE","Bias","AGB.diff")
	result$RMSE.order<-rank(result$RMSE)
	result$Method<-c("W","M")
	result$BestMod<-result$Method[result$RMSE.order==1]
	result$Sample.size<-nrow(train)
	result$RunID<-ID
	result$PlotID<-Plot
	result$Sample.strat<-strat
	result
}

#Estimate tree height using diameter (dbh) and parameters of a log-log or Weibull model (automatically detects which is which)
#From the BiomasaFP R package
#dbh = diameter
#a, b, c = parameters of Weibull/ log-log model
height.mod<-function(dbh,a,b,c=NA){
	h<-ifelse(is.na(c) & !is.na(a),exp(a+b*log(dbh/10)),a*(1-exp(-b*(dbh/10)^c)))
	h
}

############################################################################
#		Run H-D models						####
############################################################################
#Save data sheet of Height_diameter_input_data as a csv file, as read in to R as the object "dat"
#Make columns named D and H
dat$D<-dat$Diameter
dat$H<-dat$Height

#Estimate height and AGB for each tree using regional and climate based allometries
dat$Hest<-height.mod(dat$D,dat$a_par,dat$b_par,dat$c_par)
dat$AGBest<-agb.mod(dat$D,dat$Hest,dat$WD)
dat$AGBobs<-agb.mod(dat$D,dat$H,dat$WD)
dat$Hchave<-0.893-dat$E+0.76*log(dat$D/10)-0.034*((log(dat$D/10))^2)
dat$Hchave<-exp(dat$Hchave)
dat$AGBchave<-agb.mod(dat$D,dat$Hchave,dat$WD)

##########Effect of sample size##############################################

#Make vector of plot IDs
plots<-unique(dat$ID)
#Create vector of sample sizes for training data
sample.size<-seq(10,100,by=10)
#Set number of iterations to run at each sample size for each plot 
nit<-100
#Set path to write output files to
path<-"E:\\Leeds\\Height-diameter\\Xval\\"

#Loop through each plot, sample some trees, fit H-D models, test models on an independent subset of trees 
for(i in 1:length(plots)){
#Subset to data from plot i
	tmp<-dat[dat$ID==plots[i],]
	for(z in 1:length(sample.size)){
		for(u in 1:nit){
#ID vector for each row
			subs<-seq(1:nrow(tmp))
#Select some trees for training data
			train.subs<-sample(subs,sample.size[z])
#Select 50 different trees for testing data
			test.subs<-sample(setdiff(subs,train.subs),50)
#Make training and testing data subsets
			train.data<-tmp[train.subs,]
			test.data<-tmp[test.subs,]
#Fit H-D models and calculate prediction errr
			res<-run.models(train=train.data,test=test.data,Plot=plots[i],ID=paste(i,z,u,sep=""))
#Write output file (to save memory)
			write.csv(res,paste(path,i,z,u,".csv",sep=""))
		}
#Progress indicators
		cat(paste((i/length(plots))*100," % of plots"),"\n")
		cat(paste((z/length(sample.size))*100," % of sample sizes"),"\n")
		cat("\n")
	}
}


#####################Effect of different sampling strategies##########################
#Make plot id and sample size vectors
plots<-unique(dat$ID)
sample.size<-seq(10,100,by=10)
#Set number of iterations
nit<-100

#Create variable indicating the size class a tree is in
dat$Size.class<-ifelse(dat$D<200,1,
	ifelse(dat$D<300,2,
	ifelse(dat$D<500,3,4)))
#Path to write output files to
path<-"E:\\Leeds\\Height-diameter\\Xval Samp strat\\"

#Start of loop
for(i in 1:length(plots)){
	tmp<-dat[dat$ID==plots[i],]
	test.data<-tmp
	for(z in 1:length(sample.size)){
		for(u in 1:nit){

#1. Random
			subs<-seq(1:nrow(tmp))
			train.subs<-sample(subs,sample.size[z])
			train.data<-tmp[train.subs,]
			rand<-samp.strat(train=train.data,test=test.data,Plot=plots[i],ID=paste(i,z,u,sep=""),strat="Rand")

#2. Big trees only
			tmp<-tmp[order(tmp$D,decreasing=T),]
			train.data<-tmp[1:sample.size[z],]
			big<-samp.strat(train=train.data,test=test.data,Plot=plots[i],ID=paste(i,z,u,sep=""),strat="Big")

#3. Select 10 biggest, then random
			train.data<-tmp[c(1:10,sample(11:nrow(tmp),sample.size[z]-10)),]
			big.rand<-samp.strat(train=train.data,test=test.data,Plot=plots[i],ID=paste(i,z,u,sep=""),strat="Big.rand")


#4. Stratified by size class
			props<-table(tmp$Size.class)/nrow(tmp)
			sc.ss<-round(sample.size[z]*props,0)
			while(length(sc.ss)<4)sc.ss<-c(sc.ss,0)
			sc1<-tmp[tmp$Size.class==1,]
			sc2<-tmp[tmp$Size.class==2,]
			sc3<-tmp[tmp$Size.class==3,]
			sc4<-tmp[tmp$Size.class==4,]
			train.data<-rbind(sc1[sample(seq(1:nrow(sc1)),sc.ss[1]),],
			sc2[sample(seq(1:nrow(sc2)),sc.ss[2]),],
			sc3[sample(seq(1:nrow(sc3)),sc.ss[3]),],
			sc4[sample(seq(1:nrow(sc4)),sc.ss[4]),])
			strat<-samp.strat(train=train.data,test=test.data,Plot=plots[i],ID=paste(i,z,u,sep=""),strat="Strat")

#5. Biggest 10 then stratified by size class
			train.dat<-tmp[1:10,]
			props<-table(tmp$Size.class[11:nrow(tmp)])/(nrow(tmp)-10)
			sc.ss<-round((sample.size[z]-10)*props,0)
			while(length(sc.ss)<4)sc.ss<-c(sc.ss,0)
			sc1<-tmp[tmp$Size.class==1,]
			sc2<-tmp[tmp$Size.class==2,]
			sc3<-tmp[tmp$Size.class==3,]
			sc4<-tmp[tmp$Size.class==4,]
			train.dat2<-rbind(sc1[sample(seq(1:nrow(sc1)),sc.ss[1]),],
			sc2[sample(seq(1:nrow(sc2)),sc.ss[2]),],
			sc3[sample(seq(1:nrow(sc3)),sc.ss[3]),],
			sc4[sample(seq(1:nrow(sc4)),sc.ss[4]),])
			train.data<-rbind(train.dat,train.dat2)
			big.strat<-samp.strat(train=train.data,test=test.data,Plot=plots[i],ID=paste(i,z,u,sep=""),strat="Big.strat")
			res<-rbind(rand,big,big.rand,strat,big.strat)
			write.csv(res,paste(path,i,z,u,".csv",sep=""))
		}
		cat(paste((i/length(plots))*100," % of plots"),"\n")
		cat(paste((z/length(sample.size))*100," % of sample sizes"),"\n")
		cat("\n")
	}
}


